/*
  複数回のデータ取得

  \author Satofumi Kamimura
*/

#include <memory>
#include <iostream>
#include "Lidar.h"
#include "open_lidar_device.h"
#include "log_printf.h"

using namespace hrk;
using namespace std;


int main(int argc, char *argv[])
{
    auto_ptr<Lidar> lidar(open_lidar_device(argc, argv));
    if (!lidar->is_open()) {
        cerr << FILE_POSITION() << lidar->what() << endl;
        return 1;
    }

    // 計測開始
    enum { Scan_times = 10 };
    if (!lidar->start_measurement(Lidar::Distance, Scan_times)) {
        cerr << FILE_POSITION() << lidar->what() << endl;
        return 1;
    }

    vector<long> data;
    long timestamp;
    int front_index = lidar->deg2index(0);
    for (int i = 0; i < Scan_times; ++i) {
        // 計測したデータの取得
        if (!lidar->get_distance(data, &timestamp)) {
            cerr << FILE_POSITION() << lidar->what() << endl;
            return 1;
        }

        // 計測したデータの表示
        size_t n = data.size();
        cout << "n = " << n
             << ", timestamp = " << timestamp
             << ", front_data = " << data[front_index]
             << endl;
    }

    return 0;
}
