#ifndef HRK_LMS_DRIVER_H
#define HRK_LMS_DRIVER_H

/*!
  \file
  \brief LMS Sensor

  \author Satofumi Kamimura
*/

#include <memory>
#include "Lidar.h"


namespace hrk
{
    class Lms_driver : public Lidar
    {
    public:
        enum {
            Default_port = 2111,
        };

        Lms_driver();
        virtual ~Lms_driver();

        const char* what() const;

        bool open(const std::string& device_name_or_ip_address,
                  long baudrate_or_port_number, connection_t type = Serial);
        bool open(Connection* connection);

        bool is_booting();

        void close();
        bool is_open() const;
        void set_connection(Connection* connection);
        Connection* connection();

        bool start_measurement(measurement_t type,
                               int scan_times, int skip_scan);
        bool get_distance(std::vector<long>& data,
                          long *time_stamp);
        bool get_distance_intensity(std::vector<long>& data,
                                    std::vector<unsigned short>&
                                    intensity,
                                    long *time_stamp);
        bool get_multiecho(std::vector<long>& data_multi,
                           long* time_stamp);
        bool get_multiecho_intensity(std::vector<long>& data_multiecho,
                                     std::vector<unsigned short>&
                                     intensity_multiecho,
                                     long* time_stamp);
        bool set_scanning_parameter(int first_step, int last_step,
                                    int skip_step);
        void stop_measurement();

        double index2rad(int index) const;
        double index2deg(int index) const;
        int rad2index(double radian) const;
        int deg2index(double degree) const;

        double step2rad(int step) const;
        double step2deg(int step) const;
        int rad2step(double radian) const;
        int deg2step(double degree) const;

        int min_step() const;
        int max_step() const;
        int front_step() const;
        int total_steps() const;
        long min_distance() const;
        long max_distance() const;
        long scan_usec() const;
        int max_data_size() const;
        int max_echo_size() const;

        std::string sensor_series_name() const;
        std::string sensor_product_name() const;
        std::string sensor_product_version() const;
        std::string serial_id() const;

    private:
        Lms_driver(const Lms_driver& rhs);
        Lms_driver& operator = (const Lms_driver& rhs);

        struct pImpl;
        std::auto_ptr<pImpl> pimpl;
    };
}

#endif
