﻿/*!
  \file
  \brief ログ出力

  \author Satofumi Kamimura
*/

#include <cstdarg>
#include <fstream>
#include "log_printf.h"

using namespace std;


namespace
{
    string log_file_ = "error_log.txt";
    FILE* fd_ = NULL;


    bool check_is_opened()
    {
        if (!hrk::log_is_valid()) {
            return false;
        }

        if (fd_ == NULL) {
            fd_ = fopen(log_file_.c_str(), "w");
            if (fd_ == NULL) {
                return false;
            }
        }
        return true;
    }
}


void hrk::log_set_file_name(const char* file_name)
{
    if (file_name == NULL) {
        log_file_.clear();
    } else {
        log_file_ = file_name;
    }
}


bool hrk::log_is_valid()
{
    return log_file_.empty() ? false : true;
}


int hrk::log_printf(const char* format, ...)
{
    if (!check_is_opened()) {
        return -1;
    }

    va_list ap;

    // 標準エラー出力
    va_start(ap, format);
    vfprintf(stderr, format, ap);
    va_end(ap);

    // ファイル出力
    va_start(ap, format);
    int ret = vfprintf(fd_, format, ap);
    va_end(ap);

    // fclose() する機会がないので、内容を書き出す
    fflush(fd_);

    return ret;
}


int hrk::log_print(const char* text)
{
    return log_printf(text);
}


size_t hrk::log_write(const char* buf, size_t count)
{
    if (!check_is_opened()) {
        return 0;
    }

    return fwrite(buf, 1, count, fd_);
}


extern void log_printf_close()
{
    if (fd_) {
        fclose(fd_);
        fd_ = NULL;
    }
}
